function [W,OUT,C,RMSE,varargout]=ANN_REPET(IN,TARG,Nhid,ChemRAND,varargin)
//******************************************************************
//  Function to set up repetitions to train a feed-forward artificial neural network with 
//  one hidden layer. The activation function of the hidden layer is the
//  hyperbolic tangent and the identity function for the output layer.
//
//  The procedure applies a split sample test and a calibration over the whole period.
//  The best repetition is the nearest from the median simulation.
//
//  The objective function to be minimized is the Sum of Squared Errors (SSE). 
//
//  The training algorithm is Levenberg-Marquadt algorithm with bayesian regulation.
//  (see MacKay, Neural Computation, vol. 4, no. 3, 1992, pp. 415-447.
//  Foresee and Hagan, Proceedings of the International Joint Conference on Neural Networks, June, 1997.)
//
//  Input arguments of the function :
//        IN    Input data (matrix [PxN] where P is the number of input neurons and N the number of input patterns)
//        TARG  Target data (matrix [MxN] where M is the number of ouput neurons and N the number of input patterns)
//        Nhid  Number of neurons in the hidden layer
//        ChemRAND Path to the file containing the random values [0..1]
//        {opt. param. 1}  NbRepet    Number of repetitions
//        {opt. param. 2}  EPOCH      Number of traning epoch
//
//
//  Output arguments of the function :
//        W     Final weight and biais values (Matrix [NPx3]) : 3 columns col1=calib on P1, col2=calib on P2, col3 = calib
//              whole dataset.
//        OUT   network outputs (Matrix [M.N x 4] where M is the number of ouput neurons and N the number of input patterns)
//              col 1 = Calibration on P1
//              col 2 = Calibration on P2
//              col 3 = Calibration on the wole data set
//              col 4 = Validation
//
//        C     Validation criteria of the best repetition
//
//        RMSE  Matrix [4x1]  RMSE(1) Calibration performance on the first sub-period
//                            RMSE(2) Calibration performance on the second sub-period
//                            RMSE(3) Calibration performance on whole dataset      
//
//------ Validation of input variables -------
    if size(IN,2)<>size(TARG,2) then, error('Input and target data do not have the same number of patterns'); end;
    if size(TARG,1)<>1          then, error('More than one output values, not possible with this function'); end;    
    dirRAND = dir(ChemRAND)
    if dirRAND(2)==[]           then, error('No rand file at ' + ChemRAND); end;    

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined

    NbRepet   = 30;       if rhs >= 5  then,   NbRepet   = max(2,varargin(1)); end     // Number of repetitions (>=2)
    EPOCH     = 30;       if rhs >= 6  then,   EPOCH     = max(2,varargin(2)); end     // Number of epochs (>=2)
    
//------ Initialisation -------
    RAND  = fscanfMat(ChemRAND);
    if min(RAND)<0 | max(RAND)>1 then, error('RAND contains values lower than 0 or greater than 1'); end;
    
    NIn   = size(IN,1);  // Nb of inputs variables
    NOut  = size(TARG,1);// Nb of outputs variables
    NPar  = (NIn+1)*Nhid + (Nhid+1)*NOut; // Number of parameters
    
    if NPar*NbRepet>size(RAND,'*') then, error('RAND is not containing enough values'); end;
        
    Wrepet          = zeros(NPar,NbRepet,3);
    OUTrepet        = zeros(size(IN,2),NbRepet,3);
    OUT             = zeros(size(IN,2),4);
    
//------ Definition of sub periods -------
    IP1 = 1:int(size(IN,2)/2);             // First sub-period
    IP2 = int(size(IN,2)/2)+1:size(IN,2);  // Second sub-period
    IP3 = 1:size(IN,2);               // Whole dataset


//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and repetitions, selection of the best one -------
    for per = 1:3
        mprintf(' ** Period P%d **\n',per);
        // Calibration
        execstr('Ical = IP'+string(per)+';'); // selection of the sub-period
        
        [INcal_norm,INcal_norm_mean,INcal_norm_std] = ANN_NORM(IN(:,Ical),'raw2norm',2);
        IN_mean (:,per) = INcal_norm_mean;
        IN_std (:,per)  = INcal_norm_std;
        
        [TARGcal_norm,TARGcal_norm_mean,TARGcal_norm_std] = ANN_NORM(TARG(:,Ical),'raw2norm',2);
        TARG_mean (:,per) = TARGcal_norm_mean;
        TARG_std (:,per)  = TARGcal_norm_std;

        for repet =1:NbRepet
            mprintf('\tRepetition N%2d\n',repet);

            // Initial weights and biases values
            Irand_ini       = (repet-1)*NPar+1;
            Wini            = 2*RAND(Irand_ini:Irand_ini+NPar-1);       // Values between -1 and +1
            Wini            = ANN_CONV_W(Wini,NIn,Nhid,NOut,'vector');  // Conversion to get a matrix form
            Wini(:,1,[1,2]) = 0;                                        // Initial values of biases = 0
            Wini(:,:,1)     = (1/NIn)*Wini(:,:,1);                      // Limitation of variations of weights
            Wini(:,:,2)     = (1/Nhid)*Wini(:,:,2);

            // Calibration
            [Wtemp,OUTtemp,RMSEtemp]  = ANN_LMBR(INcal_norm,TARGcal_norm,Nhid,Wini,EPOCH,0,%F);           // Levenberg Marquardt calibration
            Wrepet(1:NPar,repet,per)  = ANN_CONV_W(Wtemp,NIn,Nhid,NOut,'matrix');                         // Storage of weigh values

            OUTrepet(Ical,repet,per)  = ANN_NORM(OUTtemp,'norm2raw',2,TARG_mean(:,per),TARG_std(:,per));  // Storage of final outputs (raw values)
            RMSErepet(repet,per)      = RMSEtemp;
        end
        
        OUTmed(:,per) = median(OUTrepet(:,:,per),'c'); // Calculation of the median simulation

        for repet=1:NbRepet // Calculation of criteria for all the repetitions simulations
            temp      = CRIT(OUTmed(:,per),OUTrepet(:,repet,per));
            CR(repet) = temp(1); // RMSE
        end
        
        if find(CR==0)<>[] then, mprintf('CAUTION, some repetition simulations are equal to the median simulation'); end;

        // Storage of the final parameters for the sub-period (minimal deviation from the median simulation)
        W(:,per)      = Wrepet(1:NPar,min(find(CR==min(CR))),per);
        OUT(Ical,per) = OUTrepet(Ical,min(find(CR==min(CR))),per);
        RMSE(per)     = RMSErepet(min(find(CR==min(CR))),per);
    end

    // Validation
    OUT_norm1 = ANN_SIM(ANN_NORM(IN,'raw2norm',2,IN_mean(:,1),IN_std(:,1)),Nhid,NOut,ANN_CONV_W(W(:,1),NIn,Nhid,NOut,'vector'));
    OUT_1     = ANN_NORM(OUT_norm1,'norm2raw',2,TARG_mean(:,1),TARG_std(:,1));

    OUT_norm2 = ANN_SIM(ANN_NORM(IN,'raw2norm',2,IN_mean(:,2),IN_std(:,2)),Nhid,NOut,ANN_CONV_W(W(:,2),NIn,Nhid,NOut,'vector'));
    OUT_2     = ANN_NORM(OUT_norm2,'norm2raw',2,TARG_mean(:,2),TARG_std(:,2));

    OUT(:,4)  = [OUT_2(IP1) OUT_1(IP2)]';
    C         = CRIT(TARG',OUT(:,4));

//------ Outputs -------
    if lhs>=4 then, varargout(1)=[IN_mean;IN_std]; end
    if lhs>=5 then, varargout(2)=[TARG_mean;TARG_std]; end
    if lhs>=6 then, varargout(3)=RMSErepet; end
    if lhs>=7 then, varargout(4)=Wrepet; end
    if lhs>=8 then, varargout(5)=OUTrepet; end
    
endfunction


 
