function [X,CALCcont,CRcont,varargout]=AREG_CAL_LM(Q,NPREV,varargin)
//******************************************************************
//  Function to calibrate the autoregressive ARIMA(1,2,1) model with a split sample test
//
//  Input arguments of the function :
//        Q     Discharge data (matrix [Nx1] where N is the number of time steps)
//        NPREV Vector with a list of the forecasting lead time considered
//        {opt. param. 1}  ICAL   Treshold for numerical criteria calculation
//        {opt. param. 2}  FOBJ   Transformation applied to calculate the objective function (default = 'Q', possible = 'logQ' or 'racQ')
//        {opt. param. 3}  Xini_AREG   Initial value of parameter set (matrix [2x1])
//
//  Output arguments of the function :
//        X     3 sets of parameters corresponding to calibration on P1, P2 and total period
//        CALC  Calculated outputs in calibration and validation (matrix [Nx4], col1 = cal on P1, col2 = cal on P2, col3 = cal on P3, col4 = validation on total)
//        CR    Quality criteria (matrix [5x4], col1 = cal on P1, col2 = cal on P2, col3 = cal on P3, col4 = validation on total)
//

//------ Validation of input variables -------
    if size(Q,2)<>1 | size(NPREV,2)<>1  then,  error('Wrong size of Q, P, E or NPREV vector'); end;        
    if find(Q<0) then, warning('Negative value of Q'); end
    
    
//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined

    ICAL = find(Q>=0);     if rhs >= 3 then, ICAL  = varargin(1); end;
    if min(ICAL)<1 | max(ICAL)>size(Q,1) then error('ICAL has wrong values'); end;

    FOBJ  = 'Q';            if rhs >= 4 then, FOBJ= varargin(2); end;
    if FOBJ<>'Q' & FOBJ<>'racQ' & FOBJ<>'logQ' then, error('Wrong objective function'); end

    Xini_AREG  = [0;0;0];   if rhs >= 5 then, Xini_AREG= varargin(3); end;
    if size(Xini_AREG,1)<>2 & size(Xini_AREG,2)<>1 then, error('Wrong size of Xini'); end;

    
//------ Initialisation et fonction objectif -------
    NPREV = gsort(NPREV,'r','i');
    
    // Parameters transformation
    //    Sens = 0 >> Xt 2 X
    //    Sens = 1 >> X 2 Xt
    deff('[Xout]=XTRANS(Xin,Sens)',...
    'if Sens==0 then, Xout(1)=sinh(Xin(1));Xout(2)=sinh(Xin(2));Xout(3)=sinh(Xin(3)); end;'+...
    'if Sens==1 then, Xout(1)=asinh(Xin(1));Xout(2)=asinh(Xin(2));Xout(3)=asinh(Xin(3)); end;')

    Xtini = XTRANS(Xini_AREG,1);
    
    if FOBJ=='Q'    then, deff('[TransQ]=QTRANS(Q)','TransQ=Q'); end
    if FOBJ=='racQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=abs(Q).^0.5'); end
    if FOBJ=='logQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=log(abs(Q)+%eps)'); end
        
    Nval  = size(Q,1);
    
    // Objective function (for 1 lead-time) expressed with the persistance criteria
    deff('Pers=PERSIST(Qo,Qc,L,Iok)',...
      'V1   = Qo-DECALVECT(Qc,int(L));' + ...
      'V2   = Qo-DECALVECT(Qo,int(L));' + ...
      'Pers = 100*(1 - sum(V1(Iok).^2)./sum(V2(Iok).^2));') 
    
    deff('[OBJ]=FctObjAREG_RMSE_LM(Xt,m)',...
     'X=XTRANS(Xt,0);' + ...
     'Qsim  = c_AREG(X,COL,Q);'+...
     'TSobj = DECALVECT(QTRANS(Q(IcalageAREG)),-COL);'+...
     'TScalc= QTRANS(Qsim(IcalageAREG,COL+1));'+...
     'OBJ   = TSobj-TScalc;');

//------ Definition of sub periods -------

    IP1 = ICAL(find(ICAL<=int(Nval*0.5 + 5)));   // First sub-period (P1)
    IP2 = ICAL(find(ICAL>int(Nval*0.5)));   // First sub-period (P1)

    IP3 = ICAL(find(ICAL>5));                // Whole dataset

    if IP1==[] | IP2==[] then, error('P1 or P2 is an empty period'); else mprintf('\n- %6d of values taken into account over %6d -\n',size(IP3,2),Nval); end;

//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods (for each of the forecasting lead-time) and validation -------
    NbPrev = 0;
    CALCcont(:,1) = Q;
    
    for iCOL = 1:size(NPREV,1)
        COL = NPREV(iCOL);
        for per = 1:3
            mprintf('\n ** Period P%d **\n',per);

            // Calibration
            execstr('IcalageAREG = IP'+string(per)+';'); // selection of the sub-period
            [Xtemp,Opti]  = lsqrsolve(Xtini,FctObjAREG_RMSE_LM,size(IcalageAREG,2));

            // Storage of results
            X(:,3*NbPrev+per)       = XTRANS(Xtemp,0);
	          REStemp = c_AREG(Xtemp,COL,Q);
            CALCcal(:,3*NbPrev+per) = REStemp(:,COL+1);
            CRcal(1,3*NbPrev+per)   = PERSIST(Q,REStemp(:,COL+1),COL,IcalageAREG);
            tempvect                = DECALVECT(REStemp(:,COL+1),COL);
            temp                    = CRIT(Q(IcalageAREG),tempvect(IcalageAREG));
            CRcal(2,3*NbPrev+per)   = temp(1);
        end

        // Validation (perfect rainfall)
        CALCcont(:,NbPrev+1) = [CALCcal(1:int(Nval/2),3*NbPrev+2);CALCcal(int(Nval/2)+1:$,3*NbPrev+1)];
        CRcont(1,NbPrev+1)   = PERSIST(Q(:),CALCcont(:,NbPrev+1),COL,IP3);
        tempvect             = DECALVECT(CALCcont(:,1),COL);
        temp                 = CRIT(Q(IP3),tempvect(IP3));
        CRcont(2,NbPrev+1)   = temp(1);

        NbPrev = NbPrev + 1;
        
    end
//------ Outputs -------
    if lhs>=4 then, varargout(1)=CALCcal; end
    if lhs>=5 then, varargout(2)=CRcal; end

endfunction
