function [Crit]=CRIT(OBS,CALC)
//******************************************************************
//  Function to calculate performance criteria 
//  The function calculates different criteria evaluating the distance 
//  between an observed time series and a calculated one. 
//
//  Input arguments of the function :
//        OBS  Observed data (matrix [Mx1] where M is the number of time steps)
//        CALC Calculated data (matrix [Mx1] where M is the number of time steps)
//
//  Output arguments of the function :
//        Crit Vector of calculated criteria :
//              - Crit(1) = RMSE
//              - Crit(2) = NASH
//              - Crit(3) = C2M [ =NASH(2-NASH) ]
//              - Crit(4) = Mean error CALC-OBS
//              - Crit(5) = Absolute error exceeded 20% of time

//------ Validation of input variables -------
    if size(OBS,2)<>1&size(CALC,2)<>1 then, error('OBS or CALC are not column vector'); end;
    if size(OBS,1)<>size(CALC,1)      then, error('OBS and CALC have not the same size'); end;

//------ CORE function code -------

  // RMSE
  Crit(1)=(sum((OBS-CALC).^2)/size(OBS,1))^0.5;

  // Nash Criteria
  Crit(2)=%nan;
  if sum((OBS-mean(OBS)).^2)<>0 then, Crit(2)=100*(1-sum((OBS-CALC).^2)/sum((OBS-mean(OBS)).^2)); end

  // C2M 
  Crit(3)=%nan;
  if ~isnan(Crit(2)) then, Crit(3)=Crit(2)/(200-Crit(2))*100; end
  
  // Mean error
  Crit(4)=mean(CALC-OBS);

  // Error 80%
  temp = perctl(abs(CALC-OBS),80);
  Crit(5)=temp(1);

endfunction


