function [X,CALC,CR,varargout]=GR4J_CAL_LM(Q,P,E,TS,varargin)
//******************************************************************
//  Function to calibrate the GR4J model with a split sample test
//  Optimisation routine is the Levenberg Marquardt (see lsqrsolve)
//
//  Input arguments of the function :
//        Q     Discharge data (matrix [Nx1] where N is the number of time steps)
//        P     Rainfall data (matrix [Nx1] where N is the number of time steps)
//        E     Potential evapotranspiration data (matrix [Nx1] where N is the number of time steps)
//        TS    Time step length in hours
//        {opt. param. 1}  ICAL   Indices of Q vector to use for evaluation of objective function
//        {opt. param. 2}  TSini  Length of warming-up period
//        {opt. param. 3}  FOBJ   Transformation applied to calculate the objective function (default = 'Q', possible = 'logQ' or 'racQ')
//        {opt. param. 4}  Xini   Initial value of parameter set (matrix [4x1])
//        {opt. param. 5}  RSini  Initial value of reservoirs level (matrix [2x1])
//
//  Output arguments of the function :
//        X     3 sets of parameters corresponding to calibration on P1, P2 and total period
//        CALC  Calculated outputs in calibration and validation (matrix [Nx4], col1 = cal on P1, col2 = cal on P2, col3 = cal on P3, col4 = validation on total)
//        CR    Quality criteria (matrix [5x4], col1 = cal on P1, col2 = cal on P2, col3 = cal on P3, col4 = validation on total)
//        {opt. out. 1}   S   Calculated time series of S levels (matrix [Nx2], col1 = calibration, col2=validation)
//        {opt. out. 2}   R   Calculated time series of R levels (matrix [Nx2], col1 = calibration, col2=validation)
//
//------ Validation of input variables -------
    if size(Q,1)<>size(P,1) | size(Q,1)<>size(E,1)  then,  error('Wrong size of Q, P or E vector'); end;
    if size(Q,2)<>1 | size(P,2)<>1 | size(E,2)<>1   then,  error('Wrong size of Q, P or E vector'); end;    
    if find(isnan(P))<>[] | find(isnan(E))<>[] | find(isnan(Q))<>[] then,  error('Nan values in Q, P or E vector'); end;    

    if find(P<0)<>[] | find(E<0)<>[]                then,  warning('Negative values in P or E vector, calibration may be deficient'); end;    
    
//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined

    ICAL = find(Q>=0);     if rhs >= 5 then, ICAL  = varargin(1); end;
    if min(ICAL)<1 | max(ICAL)>size(Q,1) then error('ICAL has wrong values'); end;
    if find(Q(ICAL)<0) then
        warning('Negative value of Q'); 
        //Q=c_CORRIGE(Q,2);
    end

    TSini = 365*24/TS;      if rhs >= 6 then, TSini  = varargin(2); end;

    FOBJ  = 'Q';            if rhs >= 7 then, FOBJ= varargin(3); end;
    if FOBJ<>'Q' & FOBJ<>'racQ' & FOBJ<>'logQ' then, error('Wrong objective function'); end

    //Xini  = [400;0;100;1];  if rhs >= 8 then, Xini= varargin(4); end;
    Xini_GR4J  = list([400,1000],[-2,2],[100,1000],[1,10]);    if rhs >= 8 then, Xini_GR4J= varargin(4); end;

    RSini = [0.6;0.7];      if rhs >= 9 then, RSini= varargin(5); end;
    if size(RSini,1)<> 2 & size(RSini,2)<>1 then, error('Wrong size of RSini'); end;
    if max(abs(RSini))>1 then, error('Wrong value of RSini'); end;
    
//------ Initialisation et fonction objectif -------
    // Parameters transformation
    //    Sens = 0 >> Xt 2 X
    //    Sens = 1 >> X 2 Xt
    deff('[Xout]=XTRANS(Xin,Sens)',...
    'if Sens==0 then, Xout(1)=exp(Xin(1));Xout(2)=sinh(Xin(2));Xout(3)=exp(Xin(3));Xout(4)=exp(Xin(4))+0.5; end;'+...
    'if Sens==1 then, Xout(1)=log(Xin(1));Xout(2)=asinh(Xin(2));Xout(3)=log(Xin(3));Xout(4)=log(Xin(4)-0.5); end;')

    if FOBJ=='Q'    then, deff('[TransQ]=QTRANS(Q)','TransQ=Q'); end
    if FOBJ=='racQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=abs(Q).^0.5'); end
    if FOBJ=='logQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=log(abs(Q)+%eps)'); end
        
    Nval  = size(Q,1);
    Q = [Q;Q]; // Doubling of vector to allow better initialisation
    P = [P;P];
    E = [E;E];
    
    deff('[OBJ]=FctObjGR4J_LM(Xt,m)',...
     'X=XTRANS(Xt,0);' + ...
     'Qsim=c_GR4J(TS,X,P,E,RSini);'+...
     'OBJ=Qsim(IcalageGR4J)-Q(IcalageGR4J);');

//------ Definition of sub periods -------
    I   = [ICAL ICAL+Nval];

    IP1 = I(find(I<=int(Nval*1.5 + TSini/2) & I>Nval+TSini));   // First sub-period (P1)
    IP2 = I(find(I>int(Nval*1.5 - TSini/2)+TSini));                  // Second sub-period (P2)

    IP3 = I(find(I>Nval+TSini));                // Whole dataset

    if IP1==[] | IP2==[] then, error('P1 or P2 is an empty period'); else mprintf('\n- %6d of values taken into account over %6d -\n',size(IP3,2),Nval); end;

//------ Pre filtering of paremeters -------

    // List of combinations
    listX   = COMBIN(Xini_GR4J);

    // Determination of the optimal set of initial parameters
    minSSE  = [%inf %inf %inf];
    for per=1:3
        mprintf('\n ** Pre filtering, P%d : ',per);    
        execstr('IcalageGR4J = IP'+string(per)+';');
        for np=1:size(listX,2)
            Xt  = XTRANS(listX(:,np),1);
            SSE = sum((FctObjGR4J_LM(Xt,0)).^2);
            if SSE<minSSE(per) then
                minSSE(per)  = SSE;
                Xtini(:,per) = Xt;
            end
        end
        mprintf('X1=%7.2f X2=%5.2f X3=%7.2f X4=%5.2f **\n',XTRANS(Xtini(:,per),0)');    
    end

//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and validation -------
    for per = 1:3
        mprintf('\n ** Period P%d **\n',per);
        
        // Calibration
        execstr('IcalageGR4J = IP'+string(per)+';'); // selection of the sub-period
        Xtemp  = lsqrsolve(Xtini(:,per),FctObjGR4J_LM,size(IcalageGR4J,2));
        
        // Storage of results
        X(:,per)      = XTRANS(Xtemp,0);
        [REStemp,Stemp,Rtemp] = c_GR4J(TS,XTRANS(Xtemp,0),P,E,RSini);        
        CALC(:,per)   = REStemp(Nval+1:$);
        S(:,per)      = Stemp(Nval+1:$);
        R(:,per)      = Rtemp(Nval+1:$);
        CR(:,per)     = CRIT(Q(IcalageGR4J),REStemp(IcalageGR4J));
    end

    // Validation
    CALC(:,4) = [CALC(1:int(Nval/2),2);CALC(int(Nval/2)+1:$,1)];
    CR(:,4)   = CRIT(Q(IP3),CALC(IP3-Nval,4));

//------ Outputs -------
    if lhs>=4 then, varargout(1)=S; end
    if lhs>=5 then, varargout(2)=R; end

endfunction
