#include <math.h>
#include <stdio.h>

// Size of X and V vector (WARNING : max UH ordinates = (int)(XSIZE/3)-5 !!)
#define XSIZE 1500

// Number of tolerated missing data
#define NLACMAX 5

// Number of parameters
#define	npX 4

/***********************************************
* GR4J model
*	PAS = Time step (hours)	
*
*	GR4J Parameters :
*	X[0] = Soil reservoir capacity 		(> 1mm)
*	X[1] = losses parameter 		(in [-50..50])
*	X[2] = Routing reseroir capacity	(> 1mm)
*	X[3] = Time constant			(in [0..XSIZE/3-5])
*
************************************************/

/* ************** UH subroutines ***************************** 
*	NPX = Nb of parameters
*	NH  = max length of the UH
*	C   = Time constant of UH
*/
double SS1(double I,double C)
{
	if (I<0) {return 0;}
	if (I<C) {return pow(I/C,2.5);}
	if (I>=C){return 1;}
	return 0;
}

double SS2(double I,double C)
{
	if (I<0)   {return 0;}
	if (I<=C)  {return 0.5*pow(I/C,2.5);}
	if (I<2*C) {return 1-0.5*pow(2-I/C,2.5);}
	if (I>=2*C){return 1;}
	return 0;
}

void UH1 (int NPX, int NH, double * X,double C)
{
	int i;
	for(i=0;i<NH;i++){X[NPX+i]=SS1((double)(i+1),C)-SS1((double)(i),C);}				
}

void UH2 (int NPX, int NH,double * X,double C)
{
	int i;
	for(i=0;i<2*NH;i++){X[NPX+NH+i]=SS2((double)(i+1),C)-SS2((double)(i),C);}				
}


/* ************** Core subroutine ****************************** */

void c_GR4J(long int nval,double PAS[],double Xinp[],double Pinp[],double Einp[],double Rcalc[],double Scalc[],double Qcalc[], double INI[], double ECHcalc[], double PRcalc[])
{
	int i,k,l,nH,nLAC;
	double P,E,Q,Sini,Rini; 
	double V[XSIZE],X[XSIZE];
	double ES,PS,PR,WS,S2,PERC,ECH,TP,R2,QR,QD,PERCFACTOR,pas;


	/* --- Model structure ----------------- */

	nH=(int)(XSIZE/3)-npX-1; 	// Maximum UH size

	//Time step
	pas=PAS[0];
	if((pas<24)&(floor(24/pas)!=24/pas)){for(i=0;i<nval;i++){Qcalc[i]=-9.999;return;}} 

	// Percolation factor
	if (pas>=12) {PERCFACTOR=2.25;}	// Daily time steps
	if (pas<12) 	{PERCFACTOR=4;} 	// Hourly time steps
	//sciprint("PAS=%5.2f PERCFACTOR=%5.2f\n",PAS[0],PERCFACTOR);	


	/* --- Initialisations ----------------- */
	nLAC=0;

	// Parameter vector
	for(i=0;i<npX;i++){X[i]=Xinp[i];}

	if(X[0]<1){X[0]=1;}
	if(X[1]<-50){X[1]=-50;}
	if(X[1]>50){X[1]=50;}
	if(X[2]<1){X[2]=1;}
	if(X[3]<0.5){X[3]=0.5;}
	if(X[3]>XSIZE/3-5){X[3]=floor(XSIZE/3-5);}
	for (i=npX;i<XSIZE;i++){X[i]=0;}

	if(INI[0]<=0){INI[0]=0.001;}
	if(INI[0]>1){INI[0]=1;}
	if(INI[1]<=0){INI[1]=0.001;}
	if(INI[1]>1){INI[1]=1;}
	
	
	// State variable vector
	Sini=X[0]*INI[0]; // initial values of S and R
	Rini=X[2]*INI[1];	
	V[0]=Sini;
	V[1]=Rini;
	for (i=2;i<XSIZE;i++){V[i]=0;}
	
	/*--- UH ------------------------------ */
	UH1(npX,nH,X,X[3]);
	UH2(npX,nH,X,X[3]);
	

	/* ------- Loop over the data -------- */
	for (i=0;i<nval;i++){
		P=Pinp[i];
		E=Einp[i];
		Q=0;

		/*--- Missing data management ---------------------------------*/ 
		if((P<0)&(nLAC<=NLACMAX)){P=0;nLAC++;}
		if((E<0)&(nLAC<=NLACMAX)){E=0;nLAC++;}
		if(nLAC>NLACMAX)         {V[0]=Sini;V[1]=Rini;P=0;E=0;nLAC=0;}

	
		/*--- CORE MODEL CODE -----------------------------------------*/ 
		
		// ......Production.............

		if(P>E)	{ES=0;
			WS=(P-E)/X[0];
			if(WS>=13){WS=13;}
			PS=X[0]*(1-pow(V[0]/X[0],2))*tanh(WS)/(1+V[0]/X[0]*tanh(WS));
			PR=P-E-PS;
			}

		else	{WS=(E-P)/X[0];
			if(WS>=13){WS=13;}
			ES=V[0]*(2-V[0]/X[0])*tanh(WS)/(1+(1-V[0]/X[0])*tanh(WS));
			PS=0;
			PR=0;
		}
		V[0]+=PS-ES;


		// ......Percolation.............

		S2=V[0]/pow(1+pow(V[0]/PERCFACTOR/X[0],4),0.25);
		PERC=V[0]-S2;
		V[0]=S2;
		PR+=PERC;

		// ......UH1 ....................
		
		for (k=0;k<nH-1;k++){
			V[2+k]=V[3+k]+X[npX+k]*PR;
		}
		V[2+nH-1]=X[npX+nH-1]*PR;
				

		// ......UH2 ....................
		
		for (l=0;l<2*nH-1;l++){
			V[2+nH+l]=V[3+nH+l]+X[npX+nH+l]*PR;
		}
		V[2+3*nH-1]=X[npX+3*nH-1]*PR;

		// ......Water exchange ....................
		
		ECH=X[1]*pow(V[1]/X[2],3.5);


		// ......Routing store calculation..........
		
		TP=V[1]+V[2]*0.9+ECH;		
		V[1]=0;
		if(TP>=0){V[1]=TP;}
		R2=V[1]/pow(1+pow(V[1]/X[2],4),0.25);
		QR=V[1]-R2;
		V[1]=R2;

		
		// ......Direct runoff calculation.........
		
		QD=0;
		if(V[2+nH]*0.1+ECH>0){QD=V[2+nH]*0.1+ECH;}


		// ......TOTAL STREAMFLOW..................
		
		Q=QD+QR;
	
		/*--- RESULTS ---------------- */
		Qcalc[i]=Q;
		Scalc[i]=V[0];
		Rcalc[i]=V[1];
		ECHcalc[i]=ECH;
		PRcalc[i]=PR;
		
		/*--- Missing data management (initial value of S and R)----------*/ 
		if(nLAC<=NLACMAX){Sini=V[0];Rini=V[1];}
	}

}
